import supertest from 'supertest';
import { App } from '../app';
import { HttpStatusCode } from '@constants/http.enum';
import { UserService } from '@services/user.service';

jest.mock('@config', () => ({
  KINERJA_URL: 'http://localhost:8000',
  SSO_PRIVATE_KEY: 
  `-----BEGIN RSA PRIVATE KEY-----
MIIEowIBAAKCAQEAhNfdOMRStmKXOfjPCNDFpWE9DXwj5vtKHjRzoUl3SOHn5xAJ
b7K69u/sqlmiH01yOBXVJLybmpONUqjXFXmU/k4lUQ8y+EnbVkvG55fNuHFPYjDI
/wxbbi71djWBRZTSs5ezLwEkNhY2Ii9lrYn9v59DZwMkmpGX8NYE24rJ/LNry76P
4xJ6gyyheR2dNfpQQ9xitsTdMY4qjkwJ2b/QHPbbhCsH4Qme61jxVcoYvxJoJA5u
3WXYct9QfCAjrstjiKPelsAgYcaRMqw3/ZPQyq3g/0EL/triVOUxGoEtKU0Sxpz+
Cj08EaGx62KHK9AR+Nn5PcboYToGZp9HXFV1jQIDAQABAoIBAFWnGWxQqUQPrZIj
ato0Qjuvxr27HSsBO3jeOO/1foPvpKbHIE503zf42o+721xzgwHGkfdt5KZ2HNj/
JC8khXyyxjzNn/+XH2U01rrcljgmVxjoe7B3NzyD6qCdPwyuOGyzk0TrZ324NI82
IEkUgBfdiIgS7rb6yQzv8ZHcmv/jfOuKjV7mt9FvKnQn4hu4foJ/d9G6AdQwpDQS
WYwGm1WTjza8prMxrzC5G9cs4ys6LIm3S5eyq0/M2QjffksvZPQZ6jN0WP9MFUHs
ckb28HRtHl0NatTM0DSpzlc4yQd+VWO+vxp+M3h5Y+xGAO7DUw7S9lmOmwkdBNXA
swIlO8ECgYEA4qT5N+0Ew+AwlRu2xeeHjZimLUoyhHFO6eZQ9HmE8dx8Lm5nF6hn
plGFLSyAG5a74Udja6VkhPITp5QkTunrn3R9fUMvGSsghCxXb4YstwRu7Uhz7EIs
MA7h+Zb0QZ0u/llB7J8SgBQ86Y13H6nDNcMXM1PcOGcKw2ZAAVoxtdsCgYEAlgym
j9WOZlOmrN2hQPEJn4Mw0JtIRLJe5hebnnLv5Pafe0vIOAyr2AROaERgm8mmi+EB
vEUaBp7LmzQ34ttHlI90nYmp9iGVWc7HPEfZiJjMjd9xFjljiwG2Cj1svKFNHHXD
i2a5bpjL2CH0b58ateiQ0j7MzeH/37msq3reQrcCgYAwmjwveA57isogT68kRYBW
JxQLxkuLB1lq408c/io6S5P9yQMmmaCQlA7BHzfhsrNbnhl97hvePqNkXmpi2dsD
eNGoWQ3ykcMJVRDzA4z3OmYnUFA7q/0Z2TcwdOSwZWq8dJukiIBJIw3k78dJ7PfF
wa5p/WPOXJp8SKMVJyhd3QKBgGA2AF7dyg7muErDWXAwncD/977qsm4KCTt0C76P
aC7hy+Tc9KaryUwbhsLohSGfLQEn51r22/c+vsLnuqXlgIJLcbTJ3ocJhaAuLCYE
1AD4KMEXcoWTirZJBJaQSDazHqSPBjFOTrwRkTMmonsoA5J8as/EoMGqlcy4W3Ug
tX8FAoGBAITPfq5jOkAa0iyVGDuQa91bl+35uIHlNadZn1l23PFQ/OXZBPC0lXdT
en0CjsFEXtH9fo6U+lL1dgP9ZP2EuI1SxVBvdCh0XtCdPEgxyjap0JJWGRaWLlbw
KkcqasUdKjS/z/qQxYc4FSpK9w3GDkxMQeWIiJ5wC78kdY2hHkqe
-----END RSA PRIVATE KEY-----`,
  SSO_PRIVATE_KEY_EXPIRATION: 900,
}));

jest.mock('axios', () => ({
  post: jest.fn().mockResolvedValueOnce({
    headers: { 'set-cookie': ['mockedCookie'] },
    data: { data: { redirectURL: 'http://example.com/redirected' } }
  })
}));

describe('SSO Server API Tests', () => {
  const application = new App();
  it('should be a fail login', async () => {
    const authInvalidData = {
      email: 'director@salmanitb.com',
      password: 'direct',
    };

    await supertest(application.app)
      .post('/login?serviceURL=http://localhost:8000&callbackURL=http://localhost:8000/api/v1/sign-in-from-sso')
      .send(authInvalidData)
      .expect(HttpStatusCode.Unauthorized);
  });

  it('should be a success login', async () => {
    const validAuthData = {
      user_id: 1,
      email: 'director@salmanitb.com',
      password: '$2a$12$duoDr8CNoTx9utUDTk.APur9OZg4178Al2KqBikGIt0PSYwRxadBW',
      position_id: 1,
      dept_id: 1,
      status: 1,
    };
    const mockUpdateApplication = jest.spyOn(UserService.prototype, 'getUserByEmail');
    mockUpdateApplication.mockResolvedValueOnce(validAuthData);

    // Make a POST request to simulate a login
    await supertest(application.app)
      .post('/login?serviceURL=http://localhost:8000&callbackURL=http://localhost:8000/api/v1/sign-in-from-sso')
      .send({ email: 'director@salmanitb.com', password: 'director' })
      .expect(HttpStatusCode.Found);

    // Add assertions to ensure that it redirected to the expected URL
    // Assertions for setting cookies can also be added if necessary
  });
});
