import { IUserLogin } from '@interfaces/auth.interface';
import { UserService } from './user.service';
import { compare } from 'bcryptjs';
import { Response } from 'express';
import { HttpException } from '@exceptions/http.exception';
import { HttpStatusCode } from '@constants/http.enum';
import axios from 'axios';
import { AuthHelper } from '@helpers/auth.helper';
import { KINERJA_URL } from '@config';

export class AuthService {
  private userService = new UserService();

  private allowedOrigins = {
    [KINERJA_URL]: true,
  };

  public async login(res: Response, serviceURL: string, callbackURL: string) {
    if (!serviceURL) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }

    if (!this.allowedOrigins[serviceURL]) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }

    if (!callbackURL.includes(serviceURL) || !callbackURL) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }

    return res.render("login", {
      title: "SSO Salman ITB | Login",
      error: null,
      email: null
    });
  }

  public async dologin(res: Response, userLoginData: IUserLogin, serviceURL: string, callbackURL: string) {
    if (!serviceURL) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }

    if (!this.allowedOrigins[serviceURL]) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }

    if (!callbackURL.includes(serviceURL) || !callbackURL) {
      throw new HttpException(HttpStatusCode.Forbidden, 'Access denied');
    }
    
    const user = await this.userService.getUserByEmail(userLoginData.email);
    // Check user
    if (!user) {
      throw new HttpException(HttpStatusCode.Unauthorized, 'Invalid credentials');
    }

    if (user.status !== 1) {
      throw new HttpException(HttpStatusCode.Unauthorized, 'Your account is not active');
    }

    // Check password
    const isValidPassword = await compare(userLoginData.password, user.password);
    if (!isValidPassword) {
      throw new HttpException(HttpStatusCode.Unauthorized, 'Invalid credentials');
    }

    // User is valid
    // Send to serviceURL
    const jwtToken = AuthHelper.createAccessToken(user);
    try {
      const response = await axios.post(callbackURL, 
        { }, 
        { 
          headers: {
            Authorization: `Bearer ${jwtToken}`
          }
        }
      );

      const cookies = response.headers['set-cookie'];
      for (const cookie of cookies) {
        res.set('Set-Cookie', cookie);
      }
      res.redirect(response.data.data.redirectURL);
    } catch (error) {
      throw new HttpException(HttpStatusCode.InternalServerError, 'Internal server error');
    }
  }
}
